# (c) 2020 MPIB <https://www.mpib-berlin.mpg.de/>,
#
# This file is part of castellum-scheduler.
#
# castellum-scheduler is free software; you can redistribute it and/or
# modify it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation; either version 3 of the
# License, or (at your option) any later version.
#
# Castellum is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with Castellum. If not, see
# <http://www.gnu.org/licenses/>.

from time import sleep

from django.db import DatabaseError

from .models import Job


def request(data, timeout=10, interval=1):
    job = Job.objects.create(data=data)
    t = 0
    while True:
        sleep(interval)
        t += interval
        job = Job.objects.get(pk=job.pk)

        if job.state == Job.FINISHED:
            job.delete()
            return True
        if job.state == Job.ERROR:
            job.delete()
            return False
        if t > timeout:
            job.delete()
            raise TimeoutError


def listen(interval=1):
    while True:
        for job in Job.objects.filter(state=Job.NEW):
            try:
                job.state = Job.STARTED
                job.save(force_update=True)
            except DatabaseError:
                # Job has already timed out
                continue
            yield job.pk, job.data

        sleep(interval)


def response(pk, success):
    try:
        job = Job.objects.get(pk=pk, state=Job.STARTED)
    except Job.DoesNotExist:
        raise KeyError

    job.state = Job.FINISHED if success else Job.ERROR
    try:
        job.save(force_update=True)
    except DatabaseError:
        # Job has already timed out
        pass
